import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.*;
import java.io.*;
import java.util.*;

/**
 * Fenetre qui affiche le jeu, le hero et les monstres et
 * les evenements claviers.
 *
 * @version    1.0 17 Nov 1999
 * @author     Stephane Caillette & Thomas Salamito
 */
public class FenetreJeu extends JPanel implements Runnable,KeyListener {
    /* Classe dependant des evenements claviers et affichant le jeu. */
    // attribut
    /** Variable nous precisant si le jeu est parti ou non. */
    private boolean jeu;
    /** Les evenements claviers sont pris oui ou non en compte. */
    private boolean touche ;
    /** Variable de type Level. */
    private Level level;
    /** hauteur et largeur de la fenetre de jeu. */
    private int largeur, hauteur;
    /** Nombre de monstres presents dans le jeu. */
    private int nbMonstre;
    /** Tableau gerant les differents monstres. */
    private Monstre[] tableauDeMonstre;
    /** Notre heros. */
    private Heros heros;
    /** Image sauvegardee en memoire dans db pour ne pas tout redessiner. */
    private Image db;
    /** Contexte graphique de db. */
    private Graphics dbGraphics;
    /** Le thread qui anime le jeu. */
    private Thread thread;
    /** La fenetre de notre programme. */
    private FenetrePrincipale fp;
    /** Image que l'on affiche au demarrage. */ 
    private String imageAAfficher;
    
    // constructeur
    /**
     * Construit la zone de jeu.
     * @param fp La fenetre dans laquelle se deroule le jeu.
     */ 
    public FenetreJeu(FenetrePrincipale fp) {
	largeur = Sprite.getTaille() * Level.getNbDalleX();
	hauteur = Sprite.getTaille() * Level.getNbDalleY();
	this.fp = fp;
	level = new Level(this);
	jeu = false;
	imageAAfficher = "loderunner.jpg";
	// on gere les touches du clavier
	addKeyListener(this);
	this.initialiseToi();
	
    }
    
    // methodes
    /**
     * Elle est lancee au demarrage et 
     * est identique a la methode
     * initialise() a part qu'on lui affecte un thread.
     */ 
    public void initialiseToi(){
	this.initialise();
	this.lanceThread();
    }
    
    /**
     * Elle cree un thread et 
     * le met en marche .
     */
    public void lanceThread() { 
	thread = new Thread(this);
	thread.start();
    }
    
    /**
     * Elle charge les dalles, les echelles,
     * les cadeaux et cree le heros et les monstres. 
     * On affecte l'observer FenetrePrincipale
     * a l'observable qui est notre hero.
     */
    public void initialise() {
      nbMonstre = (level.getNiveau() / 2) + 1;
      level.charge();
	heros = new Heros(Level.getPosXInitHeros() * Sprite.getTaille(), 
                          Level.getPosYInitHeros() * Sprite.getTaille(), "mario_d",this);
	heros.addObserver(fp);
        tableauDeMonstre = new Monstre[3];
        for(int i = 0 ; i < 3 ; i++) {
            tableauDeMonstre[i] = new Monstre(Level.getPosXInitMonstre(i + 1) * Sprite.getTaille(), 
                                              Level.getPosYInitMonstre(i + 1) * Sprite.getTaille(), "monstre",this);
        }
    }
    
    /**
     * Elle est appellee lorsque le heros a fini 
     * le tableau.
     */
    public void niveauSuivant() {
	level.suivant();
	this.initialise();
    }
    
    /**
     * Elle est appellee quand on a fini tous
     * les niveaux ou quand on a perdu. 
     * @param image L'image appellee si on a perdu ou fini le jeu.
     */
    public void finDuJeu(String image) {
	jeu = false;
	imageAAfficher = image;
	touche= true;
	fp.repaint();
        this.initialise();
    }
    
    /**
     * Elle est appelle quand on clique
     * dans le menu Fichier et le sous menu nouveau.
     * Elle nous place au premier level et remet a 0
     * le score, le nombre de dynamite et restaure le 
     * nombre de vies.
     */
    public void nouveau()  {
	heros.setNbVie();
	heros.setScore();
	heros.setDynamite();
	jeu =true;
	Level.setNiveau(1);
	this.initialise();
	touche = true; 
    }
    
    /**
     * Elle est appelle quand on veut arreter
     * momentanement le jeu. Si l'on veut repartir
     * on doit l'appeler une deuxieme fois.
     */
    public void pause() {
      
      if(thread.isAlive()) {
	  thread.stop();}
	else {
	    thread = new Thread(this);
	    thread.start();
	}
    }

    /**
     * Elle est definie pour que les actions 
     * des touches fonctionnent sous Windows.
     * @return true Necessaire pour que les touches fonctionnent sous Windows.
     */
    public boolean isFocusTraversable() {
	return true;
    }
    
    /** 
     * Elle sauve le fichier courant
     * pour le recharger ulterieurement.
     * @param fichierDeSauvegarde Le nom du fichier sauvegardee.
     */
  public void sauvegardeToi(String fichierDeSauvegarde) {
    if(jeu) {

      try {
	thread.sleep(5000); // stabilise le jeu
      } catch(InterruptedException e) {
	System.err.println("CLASS FenetreJeu : sleep erreur !");
	e.printStackTrace();
      }

      try {
	PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(fichierDeSauvegarde)));
	level.sauvegardeToi(pw);
	
	heros.sauvegardeToi(pw);
	
	for(int i = 0 ; i < nbMonstre ; i++) {
	  tableauDeMonstre[i].sauvegardeToi(pw);
	}
	
	pw.close();
      } catch(IOException ex) {
	System.err.println("CLASS FenetreJeu : echec lors de l'ouverture du fichier : " + fichierDeSauvegarde);
	ex.printStackTrace();
      }
    }
  }
  
  
    /** 
     * Elle charge le fichier courant.
     * @param fichierACharger c'est le nom du fichier charge.
     */
  public void chargeToi(String fichierACharger) {
    try {
      BufferedReader br = new BufferedReader(new FileReader(fichierACharger));
      StringTokenizer st = new StringTokenizer(br.readLine(), "|");
      level.chargeToi(st);
      
      heros.chargeToi(st);
      
      nbMonstre = (level.getNiveau() / 2) + 1;
      
      for(int i = 0 ; i < nbMonstre ; i++) {
	tableauDeMonstre[i].chargeToi(st);
      }
      
      br.close();
    } catch(IOException ex) {
      System.err.println("CLASS FenetreJeu : echec lors de l'ouverture du fichier : " + fichierACharger);
      ex.printStackTrace();
    }
    
    jeu = true;
  }
    
    /** 
     * Elle se charge de dessiner le decors,
     * le heros et les monstres en utilisant le double buffer.
     * @param g C'est le graphics que l'on va dessiner.
     */
    public final void paintComponent (Graphics g) {
	
	// pour que les actions du clavier fonctionnent sous Linux.
	this.requestFocus();
	// Si la variable jeu est a true.
	if(jeu == true) {
	    db = createImage(largeur, hauteur);
	    dbGraphics = db.getGraphics();
	    // Fond d'ecran noir
	    dbGraphics.setColor(Color.black);
	    // On cree dbGraphics avec un fond d'ecran noir
	    dbGraphics.fillRect(0, 0, largeur, hauteur);
	    
	    Sprite s = new Sprite(0, 0, null, this);
	    
	    for(int  i = 0 ; i < (Level.getNbDalleX() * Level.getNbDalleY()) ; i++) {
		switch(Level.getDalle((i % Level.getNbDalleX()) * Sprite.getTaille(), (i / Level.getNbDalleX()) * Sprite.getTaille())) {
		case ' ' : continue;
		case '*' : s.setImage("mur" + Level.getNiveau()); break;
		case 'C' : s.setImage("clef"); break;
		case 'B' : s.setImage("cadeau"); break;
		case 'P' : if(heros.getClef() == true) s.setImage("porte_o"); 
		else s.setImage("porte_f");  break;
		case 'H' : s.setImage("echelle"); break;
		case 'D' : s.setImage("dynamite"); break;
		default  : System.err.println("CLASS FenetreJeu : Dalle inconnue !");
		}  
		s.setX((i % Level.getNbDalleX()) * Sprite.getTaille());
		s.setY((i / Level.getNbDalleX()) * Sprite.getTaille());
		s.dessineToi(dbGraphics);
	    }
	    
	    // on dessine le heros de le meme contexte graphique que le decors.
	    heros.dessineToi(dbGraphics);
	    // on gere les interactions du heros dans le decors.
	    heros.verification();

	    // on dessine les monstres dans le decors.
	    for(int i = 0 ; i < nbMonstre ; i++) {
		tableauDeMonstre[i].dessineToi(dbGraphics);
	    }
	    // Gestion des monstres dans le decors en fonction du heros.
	    for(int i = 0 ; i < nbMonstre ; i++) {
		tableauDeMonstre[i].pousuitHeros(heros);
	    }
	    // On recopie le double buffer.
	    g.drawImage(db, 0, 0, this);
	}
	// Si la variable jeu est a faux.
	else {
	    try {
		g.drawImage(Toolkit.getDefaultToolkit().getImage("./sprite/" + imageAAfficher), 0, 0, this);
	    } catch(Exception ex) {
		System.err.println("CLASS FenetreJeu : echec lors de la lecture de l'image : " + imageAAfficher);
		ex.printStackTrace();
	    }
	}
	touche = true;
    }
    
    /**
     * Elle gere le thread qui dessine le JPanel
     */
    public void run() {
	while(true) { 
	    // On redessine les elements du jeu
	    fp.repaint();
	    try {
		//Entre chaque redessinage, on attend 500ms.
		Thread.sleep(500); 
	    } catch(InterruptedException e) {
		System.err.println("CLASS FenetreJeu : sleep erreur !");
		e.printStackTrace();
	    } 
	}
    }
    
    // accesseurs
    /**
     * La taille minimale de la fenetre de visualisation.
     * @return une fenetre (largeur * hauteur).
     */ 
    public Dimension getMinimunDimension() {
	return getPreferredSize(); 
    }
    
    /**
     * Pour que la fenetre dessinee fasse au moins une
     * taille minimale.
     * @return Dimension La taille de la fenetre.
     */
    public Dimension getPreferredSize() {
	return new Dimension(largeur,hauteur);
    }
    
    /**
     * Pour acceder au heros courant.
     * @return heros retourne le heros courant.
     */
    public Heros getHeros() {
	return heros;
    }
    
    /**
     * Gestion des evenements sur le clavier
     * @param keyEvent touche clavier utilisee
     */
    public void keyTyped(KeyEvent keyevent) {}
    
    /** 
     * Gestion des evenements sur le clavier
     * @param keyEvent touche clavier utilisee
     */
    public void keyPressed(KeyEvent keyevent) {}
    
  /** 
     * Gestion des evenements sur le clavier
     * @param keyEvent touche clavier utilisee
     */
    public void keyReleased(KeyEvent e) {
	if (touche) {
	    switch (e.getKeyCode())
		{
		case KeyEvent.VK_UP : heros.deplaceToi('h');break;
		case KeyEvent.VK_DOWN :heros.deplaceToi('b'); ;break;
		case KeyEvent.VK_LEFT : heros.deplaceToi('g');break;
		case KeyEvent.VK_RIGHT :heros.deplaceToi('d');break;
		case KeyEvent.VK_SPACE : heros.creuse() ; break;
		case KeyEvent.VK_CONTROL : heros.dynamite(); break;
		case KeyEvent.VK_ENTER  : jeu = true; break;
		} 
	    touche = false;
	}
    }
} 



