import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

/**
 * Classe qui gere notre fenetre qui contient
 * les menus, la fenetre de jeu et la fenetre
 * des observables.
 *
 * @version 1.0 17 Nov 1999
 * @author Stephane Caillette & Thomas SALAMITO
 */
public class FenetrePrincipale extends JFrame implements ActionListener,Observer {
    // attribut
    
    /** Fenetre ou se deroule le jeu. */ 
    private FenetreJeu fenetreJeu;
    /** Fenetre qui gere l'arret du programme. */
    private Quit quitDialog;
    /** Fenetre qui gere l'aide. */
    private Help helpFrame;
    /** Fenetre qui gere la version du jeu. */
    private Version versionFrame;
    /** Fenetre qui gere le chargement d'un jeu. */
    FileDialog loadFileDialog;
    /** Fenetre qui gere la sauvegarde d'un jeu. */
    FileDialog saveFileDialog;
    /** Nom du fichier sauvegarde. */
    public String savePath;
    /** Nom du fichier charge. */
    public String loadPath;  
    /** Les differents labels. */
    private JLabel score, dynamite, vie; 
       
    // constructeur
    /**
     * Prend comme argument son nom .
     */
    public FenetrePrincipale(String titre) {
	// titre de la fenetre.
	setTitle(titre);
	
	// fenetre du jeu dans la zone centrale
	fenetreJeu = new FenetreJeu(this);
	// On place fenetreJeu dans la fenetrePrincipale.
	getContentPane().add(fenetreJeu, BorderLayout.CENTER);
	
	// Les Menus
	JMenuBar menuBar = creeMenu();
	setJMenuBar(menuBar);
	quitDialog = new Quit(this,true);
	saveFileDialog = new FileDialog(this,"Sauver Jeu",FileDialog.SAVE);
	loadFileDialog = new FileDialog(this,"Charger Jeu",FileDialog.LOAD);
	
	// Le panel du nombre de dynamite, du score et du nombre de vies.
	JPanel affiche = creePanel();
	getContentPane().add(affiche,BorderLayout.SOUTH);
			
	// gestion de l'affichage.
	pack();
	setVisible(true);
	// On ne veut pas retailler la fenetre.
	setResizable(false);
	
	// gestion de la fermeture de la fenetre.
	addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		setVisible(false);
		dispose();
		System.exit(0);
	    }
    });
    }
    
    /**
     * Elle positionne la fenetre au milieu de l'ecran.
     * @param b Pour savoir si on affiche ou non.
     */
    public void setVisible (boolean b) {
        if (b) {
            Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
            int largeurEcran = screenSize.width;
            int hauteurEcran = screenSize.height;
            int largeurFrame = getSize().width;
            int hauteurFrame = getSize().height;
            int posX = (largeurEcran - largeurFrame)/2;
            int posY = (hauteurEcran - hauteurFrame)/2;
            
            setLocation(posX, posY);
        }
        super.setVisible(b);
    } 
    
    /**
     * Elle gere la creation de la barre des menus.
     * @return mb Un JMenubar.
     */ 
    public JMenuBar creeMenu() {
        JMenuBar mb = new JMenuBar();
	// Le menu Fichier
        JMenu m1 = new JMenu ("Fichier");
	// Le sous menu Nouveau.
	JMenuItem mh0 = new JMenuItem("Nouveau");
        mh0.setActionCommand("Nouveau");
        mh0.addActionListener(this);
	// Le sous menu Charger Jeu.
	JMenuItem mh1 = new JMenuItem("Charger jeu");
        mh1.setActionCommand("Charger");
        mh1.addActionListener(this);
	// Le sous menu Sauvegarder Jeu.
        JMenuItem mh2 = new JMenuItem("Sauvegarder Jeu");
        mh2.setActionCommand("Sauver");
        mh2.addActionListener(this);
	// Le sous menu Quitter.
    	JMenuItem mh3 = new JMenuItem("Quitter");
        mh3.setActionCommand("Quit");
        mh3.addActionListener(this);
        
	m1.add(mh0);
	m1.addSeparator();
        m1.add(mh1);
        m1.add(mh2);
	m1.addSeparator();
	m1.add(mh3);
	
        mb.add(m1);

        // Le menu Options.
    	JMenu menuFile = new JMenu("Options");
        // Le sous menu pause.
	JMenuItem o1 = new JMenuItem("Pause");
	o1.setActionCommand("Pause");
	o1.addActionListener(this);
        // Le sous Menu Recommencez Jeu.
	JMenuItem o2 = new JMenuItem("Recommencez Niveau");
	o2.setActionCommand("Niveau");
	o2.addActionListener(this);

	menuFile.add(o1);
	menuFile.addSeparator();
	menuFile.add(o2);
    	mb.add(menuFile); 
	
	// Le menu Aide.
    	JMenu m2 = new JMenu ("Aide");
	// Le sous menu But de Jeu.
    	JMenuItem ml0 = new JMenuItem("But de Jeu");
        ml0.setActionCommand("But");
        ml0.addActionListener(this);
	// Le sous menu Version.
        JMenuItem ml1 = new JMenuItem("Version");
        ml1.setActionCommand("Version");
        ml1.addActionListener(this);
	
    	m2.add(ml0);
    	m2.addSeparator();
    	m2.add(ml1);
    	mb.add(m2);
	
        return mb;
    }

    /**
     * Elle doit etre appele pour mettre
     * a jour le nb de dynamites (numero 1),
     * le score (numero 2), et le nb de vie (numero 3).
     * @param obs Nom de l'observable.
     * @param o Valeur de l'argument observe.
     */
    public void update (Observable obs, Object o) {
	JLabel label;
	int numero = ((Integer) o).intValue();
	
	switch (numero) {
	case 1: label = dynamite;
	    label.setText("Dynamite : " + fenetreJeu.getHeros().getDynamite());
	    break;
	case 2: label = score;
	    label.setText("Score : " + fenetreJeu.getHeros().getScore());
	    break;
	case 3: label = vie;
	    label.setText("Vie : " + fenetreJeu.getHeros().getNbVie());
	    break;
	default : System.err.println("CLASS FenetrePrincipale : label inconnu !");
	}     
    } 

    /**
     * Elle recupere les evenemnents des
     * differents menus.
     * @param e Nature de l'evenement recu.
     */
    public void actionPerformed (ActionEvent e) {
	String menuItemChoisi = e.getActionCommand();
	// Si l'option du menu presse est Quit
	if (menuItemChoisi.equals ("Quit")) {
	    quitDialog = new Quit(this,true);
	    quitDialog.setVisible(true);
	}
	else
	    // Si l'option du menu presse est Nouveau
	    if (menuItemChoisi.equals ("Nouveau")) {
		fenetreJeu.nouveau();
		score.setText("Score : 0");
		vie.setText("Vie : 3");
		dynamite.setText("Dynamite : 0");
		}
	    // Si l'option du menu presse est Recommencez Niveau.
	    if (menuItemChoisi.equals ("Niveau")) {
		fenetreJeu.initialise(); 
	    }
	    else
		// Si l'option du menu presse est Charger.
		if (menuItemChoisi.equals ("Charger")) {
		    loadFileDialog.setVisible(true);
		    if (loadFileDialog.getFile() != null) {
		      String loadPath = loadFileDialog.getDirectory() + 
			loadFileDialog.getFile();
		      fenetreJeu.chargeToi(loadPath);
		    }
		}
		else 
		    // Si l'option du menu presse est Sauver.
		    if (menuItemChoisi.equals ("Sauver")) {
			saveFileDialog.setVisible(true);
			if (saveFileDialog.getFile() != null) {
			  String savePath = saveFileDialog.getDirectory() + 
			    saveFileDialog.getFile();
			  fenetreJeu.sauvegardeToi(savePath);
			}
		    }
		    else 
			// Si l'option du menu presse est Pause.
			if (menuItemChoisi.equals ("Pause")) {
			  System.out.println ("fp");
			    fenetreJeu.pause();
			}
			else 
			    // Si l'option du menu presse est But du jeu.
			    if (menuItemChoisi.equals ("But")) {
				helpFrame = new Help(this,true);
				helpFrame.setVisible(true);
			    }
			    else 
				// Si l'option du menu presse est Version.
				if (menuItemChoisi.equals ("Version")) {
				    versionFrame = new Version(this,true);
				    versionFrame.setVisible(true);
				}
    }

    /**
     * Elle cree le panel contenant le
     * nombre de dynamites, le score et le nombre de
     * vies.
     * @return le panel des elements observes.
     */
    public JPanel creePanel() {
	JPanel p = new JPanel();
	setBackground(Color.lightGray);
	setForeground(Color.black);
	// Le JPanel cree possede 1 ligne et 3 colonnes.
	p.setLayout(new GridLayout(1,3,1,1));
	// Nouveau label score.
	score = new JLabel();
	// Nouveau label vie.
	vie = new JLabel();
	// Nouveau label dynamite.
    	dynamite = new JLabel();
	
	// On ajoute le panel a fenetrePrincipal
	getContentPane().add(p,BorderLayout.SOUTH);
	
	// chaque JLabel est constitue d'un label et d'une icone 
	dynamite.setText("Dynamite : 0");
	dynamite.setIcon(new ImageIcon ("sprite/dynamite.gif"));
	dynamite.setHorizontalAlignment(JLabel.LEFT);
	p.add(dynamite);
	score.setText("Score : 0");
	score.setIcon(new ImageIcon ("sprite/cadeau.gif"));
	score.setHorizontalAlignment(JLabel.CENTER);
	p.add(score);
	vie.setText("Vie : " + fenetreJeu.getHeros().getNbVie());
	vie.setIcon(new ImageIcon ("sprite/mario.gif"));
	vie.setHorizontalTextPosition(JLabel.LEFT);
	vie.setHorizontalAlignment(JLabel.RIGHT);
	p.add(vie);
		
	return p;
    }
    
    /**
     * Elle retourne le chemin du fichier charge.
     * @return loadPath Le chemin du fichier charge.
     */
    public String getChargePath() {
        return loadPath;
    }
    
    /** 
     * Elle retourne le chemin du fichier sauvegarde.
     * @return savePath Le chemin du fichier sauvegarde.
     */
    public String getSavePath() {
        return savePath;
    }
}




