import java.io.*;
import java.util.*;
/**
 * Classe qui gere notre heros et les interactions
 * de celui-ci avec le decors.
 *
 * @version 1.0 17 Nov 1999
 * @author Stephane Caillette & Thomas SALAMITO
 */
public class Heros extends Sprite {

    // attribut
    /** Nombre de vie du heros au debut */
    private static int nbVie = 3;
    /** Score du Heros.*/
    private static int score = 0;
    /** Boolean qui nous renseigne pour savoir si le heros a la clef. */
    private boolean clef;
    /** Direction du heros */
    private char direction;
    /** Nombre de dynamite du heros */
    private int dynamite;
    
    // constructeur
    /**
     * Le Heros sera place en (x,y) dans la fenetre de jeu.
     * Sa direction par defaut est la droite.
     * @param x Son abscisse dans le jeu.
     * @param y Son ordonnee dans le jeu.
     * @param image Sa representation dans le jeu.
     * @param f La fenetre de jeu ou il doit etre dessine.
     */
    public Heros(int x, int y, String image, FenetreJeu f) {
	super(x, y, image, f);
	dynamite =0;
	clef = false;
	direction = 'd';
    }
    
    // methodes
    /**
     * Le Heros a t-il la clef ?
     * @return oui ou non?
     */ 
    public boolean getClef() {
	return clef;
    } 

    /** 
     * Si le Heros a perdu une vie.
     */
    public void perdreUneVie() {
        if(nbVie == 0) {
            nbVie = 3;
            super.f.finDuJeu("gameover.jpg");
        }
        else {
            nbVie--; setChanged(); notifyObservers(new Integer(3));   
	    super.f.initialise();
        }
    }

    /**
     * Le Heros change de direction.
     */
    public void setDirection(char direction) {
	this.direction = direction;
    }
    
    /**
     * Combien le heros a de dynamites ?
     * @return Le nombre de dynamites.
     */
    public int getDynamite() {
	return dynamite;
    }

   /**
    * le heros a combien de vies ?
    * @return Le nombre de vies.
    */ 
    public int getNbVie()  {
	return nbVie;
    }

    /**
     * Quel est le score du heros?
     * @return Son score.
     */
    public int getScore() {
	return score;
    }

    /**
     * On remet a 0 le nombre de dynamites
     * @return 0.
     */
     public int setDynamite() {
	 dynamite = 0;
	 return dynamite;
    }

    /**
     * Restaure le nombre de vies initiale.
     * @return 3.
     */
    public int setNbVie()  {
	nbVie = 3;
	return nbVie;
    }
    
    /**
     * Remet le score a 0.
     * @return 0.
     */
    public int setScore() {
	score = 0;
	return score;
    }

    /**
     * Gere le deplacement du heros selon le decors.
     * @param direction La direction ou veut aller le heros.
     */
    public void deplaceToi(int direction) {
        if(!Level.sortieBas(super.y) &&
           Level.vide(super.x, super.y + super.TAILLE) &&
           !Level.echelle(super.x, super.y))
            super.y += super.TAILLE;
        else {
	    switch(direction) {
	    case 'd' : if(this.direction == 'g') {
                super.image = "mario_d";
                this.direction = 'd';
	    }
	    else if(!(Level.sortieDroite(super.x)) &&
                    !Level.mur(super.x + super.TAILLE, super.y))
		super.x += super.TAILLE; break;
	    case 'g' : if(this.direction == 'd') {
		super.image = "mario_g";
		this.direction = 'g';
	    }
	    else if(!(Level.sortieGauche(super.x)) &&			
                    !Level.mur(super.x - super.TAILLE, super.y))
                super.x -= super.TAILLE; break;
	    case 'h' : if(Level.echelle(super.x, super.y))
                super.y -= super.TAILLE; break;
	    case 'b' : if(!(Level.sortieBas(super.y)) &&
			  ((Level.echelle(super.x, super.y + super.TAILLE)) ||
                           !Level.mur(super.x, super.y + super.TAILLE)))
		super.y += super.TAILLE; break;
	    default : System.err.println("CLASS Heros : direction incorrecte !");
	    }
	}
    }

    /**
     * Pour sauvegarder l'objet heros.
     * @param pw printWriter vers le fichier de sauvegarde.
     */
    public void sauvegardeToi(PrintWriter pw) {
      super.sauvegardeToi(pw);
      pw.print(score + "|" + nbVie + "|" + dynamite + "|" + clef + "|" + direction + "|");
    }
    
    /**
     * Pour charger l'objet heros.
     * @param st StringTokenizer contenant les attributs du heros.
     */ 
    public void chargeToi(StringTokenizer st) {
      super.chargeToi(st);
      score = Integer.parseInt(st.nextToken()); 
      nbVie = Integer.parseInt(st.nextToken());
      dynamite = Integer.parseInt(st.nextToken());
      clef = Boolean.getBoolean(st.nextToken());
      direction =  (st.nextToken()).charAt(0);
      
    }
	
	

    /**
     * Pour que le heros creuse dans le decors
     * s'il peut le faire.
     */
    public void creuse() {
	Dalle dalle;
        if(!Level.sortieBas(super.y)) {
            if((direction == 'g') && !Level.sortieGauche(super.x))
                dalle = new Dalle(x - super.TAILLE, y + super.TAILLE);
            else if((direction == 'd') && !Level.sortieDroite(super.x))
                dalle = new Dalle(x + super.TAILLE, y + super.TAILLE);
        }
    }

    /**
     * Pour que le heros puisse poser sa dynamite dans le decors
     * s'il peut le faire.
     */
    public void dynamite() {
        Dynamite dynamite;
        if(this.dynamite > 0) {
            if(!Level.sortieBas(super.y)) {
                if((direction == 'g') && !Level.sortieGauche(super.x - super.TAILLE)) {
                    this.dynamite--; setChanged(); notifyObservers(new Integer(1));
                    dynamite = new Dynamite(x - super.TAILLE, y);
                }
                else if((direction == 'd') && !Level.sortieDroite(super.x + super.TAILLE)) {
                    this.dynamite--; setChanged(); notifyObservers(new Integer(1));
                    dynamite = new Dynamite(x + super.TAILLE, y);
                }
            }
        }
    }
    
    /**
     * Elle sert a controler toutes les interactions
     * du heros avec le tableau. Elle envoye des informations
     * a l'observer si le score, le nombre de dynamites et le 
     * nombre de vies change.
     */ 
    public void verification() {
        if(!Level.sortieBas(super.y) &&
           !Level.mur(super.x, super.y + super.TAILLE) &&
           !Level.echelle(super.x, super.y + super.TAILLE) &&
           !Level.echelle(super.x, super.y))
            super.y += super.TAILLE;
        else {
            if(Level.cadeau(super.x, super.y)) {
                Level.faireUnTrou(super.x, super.y);
                score += 100; setChanged(); notifyObservers(new Integer(2));
            } else {
                if(Level.clef(super.x, super.y)) {
                    Level.faireUnTrou(super.x, super.y);
                    clef = true;
                } else {
                    if(Level.dynamite(super.x, super.y)) {
                        Level.faireUnTrou(super.x, super.y);
                        dynamite += 1; setChanged(); notifyObservers(new Integer(1));
                    } else {
                        if(Level.porte(super.x,super.y)) {
                            if(clef) super.f.niveauSuivant();
                        } else {
                            if(Level.mur(super.x, super.y)) {
                                this.perdreUneVie();
                            }
                        }
                    }
                }
            }
        }
    }
}

    
   
	 
    

