import java.io.*;
import java.util.*;

public class Level{
    
    // attribut
    // On charge toutes les dalles du decors dans un tableau.
    private static char[] tableauDeDalle;
    // On charge les positions du hero et du monstre dans un autre tableau.
    private static int[] tableauDePosInit;
    // Nombre de dalles en abscisse par tableau.
    private final static int NB_DALLE_X = 15;
    // Nombre de dalles en ordonnee par tableau.
    private final static int NB_DALLE_Y = 20;
    // Necessaire pour lire notre fichier de donnees.
    private StringTokenizer st;
    // Fenetre de jeu courante.
    private FenetreJeu f;
    // Level ou niveau courant.
    private static int niveau;

    // constructeur
    /**
     * Preparation du chargement du tableau.
     * @param f la fenetre de jeu courante.
     */
    public Level(FenetreJeu f) {
	this.f = f;
        niveau = 1;
	tableauDeDalle = new char[NB_DALLE_X * NB_DALLE_Y];
	tableauDePosInit = new int[8]; // -> couple(x, y) * (nb monstre max + heros) 
    }
    
    // methode
    /**
     * On charge le tableau a partir de notre fichier de donnees.
     */ 
    public void charge() {
	try {
	    BufferedReader br = new BufferedReader(new FileReader("./level/level" + niveau + ".dat"));
	    
	    for(int y = 0 ; y < NB_DALLE_Y ; y++) {
		for(int x = 0 ; x < NB_DALLE_X ; x++) {
		    tableauDeDalle[x + (y * NB_DALLE_X)] = (char) br.read();
		}
		br.read();
	    }
	    
	    st = new StringTokenizer(br.readLine(), ",");
	    for(int i = 0 ; i < 8 ; i++) {
		tableauDePosInit[i] = Integer.parseInt(st.nextToken());
	    }
	    
	    br.close();
	}
	catch(IOException e) {
	    System.err.println("CLASS Level : Ouverture du fichier level" + niveau + ".dat impossible !");
	    e.printStackTrace();
	}
    }
    /** 
     * Pour changer de niveau si on peut. Sinon,
     * on arrive a la fin du jeu.
     */ 
     public void suivant() {
        if(++niveau > 4) {
            niveau = 1;
            f.finDuJeu("endgame.jpg");
        }
     }

  /**
   * Pour sauvegarder l'objet heros.
   * @param pw printWriter vers le fichier de sauvegarde.
   */
  public void sauvegardeToi(PrintWriter pw) {
    pw.print(niveau + "|");
    for(int  i = 0 ; i < (NB_DALLE_X * NB_DALLE_Y) ; i++) {
      pw.print(tableauDeDalle[i] + "|");
    }
  }

  /**
   * Pour charger l'objet heros.
   * @param st StringTokenizer contenant les attributs du heros.
   */ 
  public void chargeToi(StringTokenizer st) {
    niveau  = Integer.parseInt(st.nextToken());
    for(int  i = 0 ; i < (NB_DALLE_X * NB_DALLE_Y) ; i++) {
      tableauDeDalle[i] = (st.nextToken()).charAt(0);
    }
  }
    
    /**
     * Nous dit si l'on va sortir du tableau par le bas.
     * @param y Ordonnee courante.
     */
    public static boolean sortieBas(int y) {
        return ((y / Sprite.getTaille()) == (NB_DALLE_Y - 1));
    }
    
    /**
     * Nous dit si l'on va sortir du tableau par la gauche.
     * @param x Abscisse courante.
     */
    public static boolean sortieGauche(int x) {
        return ((x / Sprite.getTaille()) == 0);
    }
    
    /**
     * Nous dit si l'on va sortir du tableau par la droite.
     * @param x Abscisse courante.
     */
    public static boolean sortieDroite(int x) {
        return ((x / Sprite.getTaille()) == (NB_DALLE_X - 1));
    }
    
    /**
     * Retourne la dalle courante.
     * @param x Abscisse courante.
     * @param y Ordonnee courante.
     */
    public static char getDalle(int x, int y) {
        return tableauDeDalle[((y / Sprite.getTaille()) * NB_DALLE_X + (x / Sprite.getTaille()))];
    }
    
    /**
     * Si la dalle est le vide. 
     * @param x Abscisse courante.
     * @param y Ordonnee courante.
     */
    public static boolean vide(int x, int y) {
        return getDalle(x, y) == ' ';
    }
    
    /**
     * Si la dalle est un mur. 
     * @param x Abscisse courante.
     * @param y Ordonnee courante.
     */
    public static boolean mur(int x, int y) {
        return getDalle(x, y) == '*';
    }

    /**
     * Si la dalle est une echelle.
     * @param x Abscisse courante.
     * @param y Ordonnee courante.
     */
    public static boolean echelle(int x, int y) {
        return getDalle(x, y) == 'H';
    }

     /**
     * Si la dalle est un cadeau.
     * @param x Abscisse courante.
     * @param y Ordonnee courante.
     */
    public static boolean cadeau(int x, int y) {
        return getDalle(x, y) == 'B';
    }

    /**
     * Si la dalle est une dynamite.
     * @param x Abscisse courante.
     * @param y Ordonnee courante.
     */
    public static boolean dynamite(int x, int y) {
        return getDalle(x, y) == 'D';
    }
    
    /**
     * Si la dalle est une clef.
     * @param x Abscisse courante.
     * @param y Ordonnee courante.
     */
    public static boolean clef(int x, int y) {
        return getDalle(x, y) == 'C';
    }

    /**
     * Si la dalle est une porte.
     * @param x Abscisse courante.
     * @param y Ordonnee courante.
     */
    public static boolean porte(int x, int y) {
        return getDalle(x, y) == 'P';
    }

    /**
     * Renvoie le nombre de dalles en abscisse.
     * @return NB_DALLE_X Nombre de dalles en abscisse.
     */
    public static int getNbDalleX() {
        return NB_DALLE_X;
    }
    
    /**
     * Renvoie le nombre de dalles en ordonnee.
     * @return NB_DALLE_Y Nombre de dalles en ordonnees.
     */
    public static int getNbDalleY() {
        return NB_DALLE_Y;
    }
    
    /**
     * La position du heros en abscisse.
     * @return tableauDePosInit[0] Position du heros en abscisse.
     */
    public static int getPosXInitHeros() {
        return tableauDePosInit[0];
    }
    
    /**
     * La position du heros en ordonnee.
     * @return tableauDePosInit[0] Position du heros en ordonnee.
     */
    public static int getPosYInitHeros() {
        return tableauDePosInit[1];
    } 

    /**
     * La position des monstres en abscisse.
     * @return tableauDePosInit[(2 * numero) + 1] Position des monstres en abscisse.
     */
    public static int getPosXInitMonstre(int numero) {
        return tableauDePosInit[2 * numero];
    }

    /**
     * La position des monstres en ordonnee.
     * @return tableauDePosInit[(2 * numero) + 1] Position des monstres en ordonnee.
     */
    public static int getPosYInitMonstre(int numero) {
        return tableauDePosInit[(2 * numero) + 1];
    }

    /**
     * Le niveau courant.
     * @return niveau Le niveau courant.
     */
    public static int getNiveau()  {
        return niveau;
    }

    /**
     * Modifier le niveau courant.
     * @param level Le niveau que l'on souhaite.
     * @return niveau Le nouveau niveau courant.
     */
    public static int setNiveau(int level)  {
        niveau = level;
	return niveau;
    }
    
    // modificateur
    /**
     * Modifie la dalle courante.
     * @param x Abscisse courante.
     * @param y Ordonnee courante.
     */
    public static void setDalle(int x, int y, char c) {
        tableauDeDalle[((y / Sprite.getTaille()) * NB_DALLE_X + (x / Sprite.getTaille()))] = c;
    }

    /**
     * Place la dynamite en position courante.
     * @param x Abscisse courante.
     * @param y Ordonnee courante.
     */
    public static void placerDynamite(int x, int y) {
        setDalle(x, y, 'X');
    }

    /**
     * Place le mur en position courante.
     * @param x Abscisse courante.
     * @param y Ordonnee courante.
     */
    public static void placerMur(int x, int y) {
        setDalle(x, y, '*');
    }

    /**
     * Met un trou en position courante.
     * @param x Abscisse courante.
     * @param y Ordonnee courante.
     */
    public static void faireUnTrou(int x, int y) {
        setDalle(x, y, ' ');
    }
  
    
}




