import java.io.*;
import java.util.*;

/**
 * Classe qui gere nos monstres et les interactions
 * de ceux-ci avec le decors.De plus, on leur attribue
 * une intelligence artificielle pour tenter d'attraper
 * le heros.
 *
 * @version 1.0 17 Nov 1999
 * @author Stephane Caillette & Thomas SALAMITO
 */
public class Monstre extends Sprite {
    
    // attribut
    /** La direction que suit le monstre. */
    private char direction;
    /** Sa position au demarrage. */ 
    private int xDepart, yDepart;

    
    // constructeur
    /**
     * Le monstre sera place en (x,y) dans la fenetre de jeu.
     * Sa direction par defaut est la droite.
     * @param x Son abscisse dans le jeu.
     * @param y Son ordonnee dans le jeu.
     * @param image Sa representation dans le jeu.
     * @param f la fenetre de jeu ou il doit etre dessine.
     */
    public Monstre(int x, int y, String image, FenetreJeu f) {
	super(x, y, image, f);
	xDepart = x;
	yDepart = y;
	direction = 'd';
    }
    
    // methode
    
    /**
     * Pour sauvegarder l'objet monstre.
     * @param pw printWriter vers le fichier de sauvegarde.
     */  
  public void sauvegardeToi(PrintWriter pw) {
    super.sauvegardeToi(pw);
    pw.print(direction + "|" + xDepart + "|" + yDepart + "|");
  }
    
    /**
     * Pour charger l'objet monstre.
     * @param st StringTokenizer contenant les attributs du monstre.
     */ 
  public void chargeToi(StringTokenizer st) {
    super.chargeToi(st);
    direction  = (st.nextToken()).charAt(0);
    xDepart = Integer.parseInt(st.nextToken());
    yDepart = Integer.parseInt(st.nextToken());
  }

    /**
     * Gere les interactions du monstre avec le decors.
     * Gere la mort des monstres si on est recouvert par une dalle.
     * Gere la poursuite du heros par le monstre de maniere assez
     * intelligente.
     * @param heros Le heros que l'on poursuit.
     */ 
    public void pousuitHeros(Heros heros) {
	// Si on est recouvert par une dalle on meurt.
	if(Level.mur(super.x, super.y)) {
	    super.x = xDepart; super.y = yDepart;
	} else {
	    // Si on est pas sur une case, on tombe.
	    if(!Level.sortieBas(super.y) &&
	       !Level.mur(super.x, super.y + super.TAILLE) &&
	       !Level.echelle(super.x, super.y + super.TAILLE) &&
	       !Level.echelle(super.x, super.y)) {
		super.y += super.TAILLE;
	    } else {
		// Au meme niveau que le heros, on l'attaque.
		if(super.y == heros.getY()) {
		    if(((super.x - heros.getX()) > 0)  && !Level.mur(super.x - super.TAILLE, super.y)) {
			super.x -= super.TAILLE; direction = 'g';
		    } else {
			if (((super.x - heros.getX()) < 0) && !Level.mur(super.x + super.TAILLE, super.y))
			    {
				super.x += super.TAILLE; direction = 'd';
			    }
			else if(super.x == heros.getX()) heros.perdreUneVie();
		    }
		} else {
		    // Si on est au dessus du heros et qu'on rencontre une echelle
		    if((super.y < heros.getY()) && Level.echelle(super.x, super.y + super.TAILLE)) {
			super.y = super.y + super.TAILLE;
		    } else {
			// Si on est en dessous du heros et qu'on rencontre une echelle
			if((super.y > heros.getY()) && Level.echelle(super.x, super.y)) {
			    super.y = super.y - super.TAILLE;
			} else {
			    if(direction == 'g') { // Sinon on va a gauche
				if(!Level.sortieGauche(super.x) && !Level.mur(super.x - super.TAILLE, super.y))
				    super.x = super.x - super.TAILLE;
				else
				    direction = 'd'; // Rate demi-tour droite !
			    } else {             // Sinon on va a droite
				if(!Level.sortieDroite(super.x) && !Level.mur(super.x + super.TAILLE, super.y))
				    super.x = super.x + super.TAILLE;
				else
				    direction = 'g'; // Rate demi-tour gauche !
			    }
			}
		    }
		}
	    }
	}
    }
}

    



