import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.image.ImageProducer;

/**
 * Classe qui gere les personnages animes.
 * Elle est observable.
 *
 * @version 1.0 17 Nov 1999
 * @author Stephane Caillette & Thomas SALAMITO
 */
public class Sprite extends Observable {

  // attribut 
  /** x et y sont les coordonnees du sprite */ 
  protected int x,y;
  /** Representation du sprite dans le jeu. */
  protected String image;
  /** La fenetre de jeu ou il doit etre dessine. */
  protected FenetreJeu fj;
  /** La taille d'un sprite. */
  protected static final int TAILLE = 32;
  /** On cree une hashtable dans laquelle on met les personnages animees. */
  private static Hashtable hashtable = new Hashtable();

  // constructeur
  /**
   * Le sprite sera place en (x,y) dans la fenetre de jeu.
   * @param x Son abscisse dans le jeu.
   * @param y Son ordonnee dans le jeu.
   * @param image Sa representation dans le jeu.
   * @param f La fenetre de jeu ou il doit etre dessine.
   */
  public Sprite(int x, int y, String image, FenetreJeu fj) {
    this.x = x;
    this.y = y;
    this.image = image;
    this.fj = fj;
  }
  
  // methodes
  /** 
   * Pour dessiner les sprites.
   * @param g On y dessine les sprites en (x, y).
   */ 
  public void dessineToi(Graphics g) {
    Image sprite = null;

    if(hashtable.containsKey(image))
      sprite = (Image) hashtable.get(image);
    else {
      try {
	sprite = Toolkit.getDefaultToolkit().createImage((ImageProducer) (getClass().getResource("./sprite/" + image + ".gif")).getContent());
	hashtable.put(image,sprite);
      } catch(Exception ex) {
	System.err.println("CLASS Sprite : echec lors de la lecture du sprite : " + image);
	ex.printStackTrace();
      }
    }
    
    g.drawImage(sprite, x, y, fj);
  }

  /**
   * Pour sauvegarder le sprite
   * @param pw PrintWriter vers le fichier de sauvegarde.
   */
  public void sauvegardeToi(PrintWriter pw) {
    pw.print(x + "|" + y + "|" + image + "|");
  }

  /**
   * Pour charger le sprite
   * @param st StringTokenizer contenant les attributs du sprite.
   */   
  public void chargeToi(StringTokenizer st) {
    x = Integer.parseInt(st.nextToken()); y = Integer.parseInt(st.nextToken()); image = st.nextToken();
  }
  
  // modificateurs
  /**
   * modificateur de l'abscisse du sprite.
   * @param x La nouvelle abscisse.
   */ 
  public void setX(int x) {
    this.x = x;
  }

  /**
   * modificateur de l'ordonnee du sprite.
   * @param y La nouvelle ordonnee.
   */ 
  public void setY(int y) {
    this.y = y;
  }

  /**
   * modificateur de l'image.
   * @param image La nouvelle Image.
   */
  public void setImage(String image) {
    this.image = image;
  }

  // accesseurs
  /**
   * Accesseur de l'abscisse occupe par le sprite.
   * @return son abscisse.
   */ 
  public int getX() {
    return x;
  }

  /**
   * Accesseur de l'ordonnee occupe par le sprite.
   * @return son ordonnee.
   */ 
  public int getY() {
    return y;
  }

  /**
   * Accesseur de l'image du sprite (sa nature).
   * @return L'image du sprite.
   */ 
  public String getImage() {
    return image;
  }

  /**
   * Accesseur de la taille des sprites.
   * @return La taille du sprite.
   */ 
  public static int getTaille() {
    return TAILLE;
  }

}

